<?php
namespace Teknisa\DF\Services;

use Teknisa\Libs\Service\Crud;
use Teknisa\Libs\Util\Environment;
use Teknisa\DF\Factories\VencimentoFactory;

class VencimentoService
{

    /**
     *  Observações: Para evitar repetição de código as funções que montam as parcelas 
     *  recebem um array de campos para diferenciar de acordo com vencimento para entrada/AF
     */

    protected $crudService;
    protected $environment;
    protected $nameProvider;
    protected $factory;

    public function __construct(Crud $crudService, Environment $environment, $nameProvider, VencimentoFactory $factory) {
        $this->crudService  = $crudService;
        $this->environment  = $environment;
        $this->nameProvider = $nameProvider;
        $this->factory      = $factory;
    }
    
    public function criaVencimentos($idEntrSaid, $clieForn, $vrPagto, $dtEmissao, $dtEntrada, $devolucao = false, $cdfilemit = '', $cdfilial = '', $nrlanctonf = '', $af = array()) {
        $nrDias = 0;
        $dtBase = '';
        $dtAux  = ''; 
        $nDias = '';
    
        $parcelas = array();
        $dados = array();
        
        if($idEntrSaid == 'E' || $devolucao) {
            
            $dadosPag = $this->factory->getPagamentoForn($clieForn);
            $formasPagamento = $this->factory->getFormasPagamento($clieForn);
            
            if(!$dadosPag || empty($formasPagamento)) {
                return null;
            }
            
            if($dadosPag["IDTPFORPGTFO"]) {
                if($dtEmissao || $devolucao) {
                    $this->numeroDias($dadosPag, $dadosPag["IDTIPODTPGTO"], $idEntrSaid, $nDias, $dtBase, $dtAux, $dtEmissao, $dtEntrada, $devolucao, $dadosPag["IDREFECALCPARC"]);
                }
            }
            else {
                $parcelas[] = array(
                    "NRDIASPARCELA"  => 0,
                    "DTPARCELA"     => $dtEmissao,
                    "VRPARCELA"     => $vrPagto
                ); 
            }
            
            $campos   = array("NRPARCELA" => "NRPARCPGTOFO", "PERCENTUAL" => "VRPERCPGTOFO", "DIAS" => "NRDIASPGTOFO");
            if($dadosPag["IDTPFORPGTFO"] == 'F') {
                $formaPagamento = $formasPagamento[0];
                $parcelas = $this->buildParcelasPagamentoFixo($formaPagamento, $dtBase, $dtAux, $nDias, $vrPagto, $campos);
            }
            else if($dadosPag["IDTPFORPGTFO"] == 'V') {
                $parcelas = $this->buildParcelasPagamentoVariavel($formasPagamento, $dtBase, $dtAux, $nDias, $vrPagto, $cdfilemit, $dtEmissao, $campos);
            }
        }
        else if($idEntrSaid == 'S') {
            // TODO IMPLEMENTAR
        }
        else if($idEntrSaid == 'A') {
            $autoForne = $this->factory->getAutoforne($af["CDFILDEST"], $af["CDFILLANC"], $af["NRAUTOFORN"]);
            $formasPagamento = $autoForne;
            $autoForne = $autoForne[0];

            if($dtEmissao) {
                $this->numeroDias($autoForne, $autoForne["IDTIPODTPGTO"], $idEntrSaid, $nDias, $dtBase, $dtAux, $dtEmissao, $dtEntrada, false, $autoForne["IDREFECALCPARC"]);
            }
            
            $campos = array("NRPARCELA" => "NRPARCPGTOAF", "PERCENTUAL" => "VRPERCPGTOAF", "DIAS" => "NRDIASPGTOAF");
            
            if($autoForne["IDTPFORPGTAF"]) {
                if($autoForne["IDTPFORPGTAF"] == 'F') {
                    $formaPagamento = $formasPagamento[0];
                    $parcelas = $this->buildParcelasPagamentoFixo($formaPagamento, $dtBase, $dtAux, $nDias, $vrPagto, $campos);
                }
                else if($autoForne["IDTPFORPGTAF"] == 'V') {
                    $parcelas = $this->buildParcelasPagamentoVariavel($formasPagamento, $dtBase, $dtAux, $nDias, $vrPagto, $cdfilemit, $dtEmissao, $campos);
                }
            }
        }
        else if($idEntrSaid == 'P') {
            // TODO IMPLEMENTAR
        }
        return $parcelas;
    }
    
    /**
    * Funções que montam as parcelas baseado na parametrização
    */
    protected function buildParcelasPagamentoFixo($formaPagamento, $dtBase, $dtAux, $nDias, $vrPagto, $campos) {
        $parcelas = array();

        $nrParcelas = $formaPagamento[$campos["NRPARCELA"]];

        $valorParcela = round($vrPagto / $nrParcelas, 2);

        $totalParcelas = 0;
        if($dtBase) {
            $dtAux = $this->adicionaDiasData($dtBase, $nDias); 
        }
        
        for($i = 1; $i <= $nrParcelas; $i++) {
            $totalParcelas += $valorParcela;
            if($i == $nrParcelas) {
                $valorParcela += ($vrPagto - $totalParcelas);
            }
            $parcelas[] = array(
                "NRDIASPARCELA" => $this->getDiferenciaDias($dtBase, $dtAux),
                "DTPARCELA"     => $dtAux,
                "VRPARCELA"     => $valorParcela
            );
            $dtAux = substr($dtAux, 0, 3) . $this->getProximoMes(substr($dtAux, 3, 2)) . substr($dtBase, 5, 5);
        }
        
        return $parcelas;
    }
    
    protected function buildParcelasPagamentoVariavel($formasPagamento, $dtBase, $dtAux, $nDias, $vrPagto, $cdfilemit, $dtEmissao, $campos) {
        $parcelas   = array();
        $totalParcelas = 0;
        
        $parcAux    = 0;
        $oldNDias   = $nDias;
        $nParcela   = 1;
        
        $campoPercentual = $campos["PERCENTUAL"];
        $campoDias       = $campos["DIAS"];
        foreach($formasPagamento as $formaPagamento) {
            $percentual = $formaPagamento[$campoPercentual] ? $formaPagamento[$campoPercentual] : 100; 
            $valorParcela =  $vrPagto * ($percentual/100);
            $diasParcela  = $nDias + $formaPagamento[$campoDias];
            
            if($dtBase) {
                $dtParcela = $this->adicionaDiasData($dtBase, $diasParcela); 
            }
            
            // Consulta Feriados
            if($formaPagamento["IDCONSDIAUTIL"] == 'S') {
                if( $formaPagamento["IDTIPODTPGTO"] == 'EMEX' || 
                    $formaPagamento["IDTIPODTPGTO"] == 'ENEX' ||
                    $formaPagamento["IDTIPODTPGTO"] == 'DSEX') {
                        $ferifeli = $this->factory->getFeriFili($cdfilemit, $this->adicionaDiasData($dtEmissao, 1), $dtParcela);
                    }
                else {
                    $ferifeli = $this->factory->getFeriFili($cdfilemit, $dtEmissao, $dtParcela);
                }
                
                if($ferifeli) {
                    if($ferifeli["NRDIAS"] > 0) {
                        $this->getDataFeriado($ferifeli, $dtParcela, $cdfilemit);
                    }
                }
            }
            // Rafael disse que essa linha serveria apenas para notas de serviço
            // Porém, para manter a compatibiidade com o delphi mantive a rotina em 29/04/2020
            // Caso seja necessario remover basta comentar a linha
            $this->diaPgto($formaPagamento["CDFORMPGTO"], $dtAux, $formaPagamento["IDTPPERIODO"]);
            
            $totalParcelas += $valorParcela;
            if($nParcela == count($formasPagamento)) {
                $valorParcela += ($vrPagto - $totalParcelas);
            }
            
            $parcelas[] = array(
                "NRDIASPARCELA" => $this->getDiferenciaDias($dtBase, $dtAux),
                "DTPARCELA"     => $dtParcela,
                "VRPARCELA"     => $valorParcela
            );
        }
        
        return $parcelas;
    }
    
    
    /**
     * Funções para obter a data para o vencimento
     */
    protected function getDataReferencia($dtBase, $idrefecalcparc) {
        if(gettype($dtBase) == "string") {
            $dtBaseDateTime = $this->getDateFromString($dtBase);
        }
        $data = '';
        if($idrefecalcparc == 'FORA_SEMANA') {
            $diaSemana  = 8 - $this->getDiaSemana($dtBase);
            $data       = $this->adicionaDiasData($dtBaseDateTime, $diaSemana);
        }
        else if($idrefecalcparc == 'FORA_DEZENA') {
            $dia = substr($dtBase, 0, 2);
            if($dia >= "01" && $dia <= "10") {
                $data = "11" . substr($dtBase, 2, 8);
            }
            else if($dia >= "11" && $dia <= "20") {
                $data = "21" . substr($dtBase, 2, 8);
            }
            else if($dia >= "21" && $dia <= "31") {
                $data = "01/" . $this->getProximoMes(substr($dtBase, 3, 2)) . substr($dtBase, 5, 5);
            }
        }
        else if($idrefecalcparc == 'FORA_QUINZENA') {
            $dia = substr($dtBase, 0, 2);
            if($dia >= "01" && $dia <= "15") {
                $data = "16" . substr($dtBase, 2, 8);
            }
            else if($dia >= "16" && $dia <= "31") {
                $data = "01/" . $this->getProximoMes(substr($dtBase, 3, 2)) . substr($dtBase, 5, 5);
            }
        }
        else if($idrefecalcparc == 'FORA_MES') {
            $data = "01/" . $this->getProximoMes(substr($dtBase, 3, 2)) . substr($dtBase, 5, 5);
        }
        else {
            $data = $dtBase;
        }
        return $data;
    }
    
    protected function numeroDias($dadosPag, $idtipodtpgto, $idEntrSaid, &$nDias, &$dtBase, &$dtAux, $dtEmissao, $dtEntrada, $devolucao, $refCalcParc, $af = false) {
      
        if($idEntrSaid == 'E' || $devolucao) {
            if($dadosPag["IDTPFORPGTFO"]) {
                if($dadosPag["IDTPFORPGTFO"] == 'V') {
    
                    $this->trataTipoDtPgto($idtipodtpgto, $nDias, $dtBase, $dtEntrada, $dtEmissao);
      
                    $dtAux = $this->getDataReferencia($dtBase, $refCalcParc);
                    $dtAux = $this->adicionaDiasData($dtAux, $nDias);
                    $dtAux = $this->adicionaDiasData($dtAux, 1, true);
                }
                else {
                    $dtBase = $dtEmissao;
                    $mes = substr($dtBase, 3, 2);
                    $ano = substr($dtBase, 6, 4);
                    if($dadosPag["NRDIAPGFIXFO"] > date('dd')) {
                        if(checkdate($mes, $dadosPag["NRDIAPGFIXFO"], $ano)) {
                            $dtAux = str_pad($dadosPag["NRDIAPGFIXFO"], 2, "0", STR_PAD_LEFT) .  substr($dtBase, 2, 8);
                        }
                        else {
                            $dtAux = $this->getUltimoDiaMes($mes, $ano) .  substr($dtBase, 2, 8);
                        }
                    }
                    else {
                        if(checkdate($mes, $dadosPag["NRDIAPGFIXFO"], $ano)) {
                            $dtAux = str_pad($dadosPag["NRDIAPGFIXFO"], 2, "0", STR_PAD_LEFT) . '/' . $this->getProximoMes(substr($dtBase, 3, 2)). substr($dtBase, 5, 5);
                        }
                        else {
                            $dtAux = $this->getUltimoDiaMes($mes, $ano) .  substr($dtBase, 2, 8);
                        }
                    }
                }
                if($dtAux && $dtBase) {
                    $nDias = $this->getDiferenciaDias($dtBase, $dtAux);
                }
                else {
                    $nDias = 0;
                }
            }
        }
        else if($idEntrSaid == 'S') {
            
        }
        // Autorização de Fornecimento
        else if($idEntrSaid == 'A') {
            if($dadosPag["IDTPFORPGTAF"]) {
                if($dadosPag["IDTPFORPGTAF"] == 'V') {
                    $this->trataTipoDtPgto($idtipodtpgto, $nDias, $dtBase, $dtEntrada, $dtEmissao);
                    $dtAux = $this->getDataReferencia($dtBase, $refCalcParc);
                    $dtAux = $this->adicionaDiasData($dtAux, $nDias);
                    $dtAux = $this->adicionaDiasData($dtAux, 1, true);
                }
                else {
                    $dtBase = $dtEmissao;
                    if($dadosPag["IDUTDIAPGTAF"] == "I" && $dadosPag["NRDIAPGFIXAF"] > date('dd')) {
                        if(checkdate($mes, $dadosPag["NRDIAPGFIXAF"], $ano)) {
                            $dtAux = str_pad($dadosPag["NRDIAPGFIXAF"], 2, "0", STR_PAD_LEFT) .  substr($dtBase, 2, 8);
                        }
                        else {
                            $dtAux = $this->getUltimoDiaMes($mes, $ano) .  substr($dtBase, 2, 8);
                        }
                    }
                    else {
                        $dia =  substr($dtBase, 0, 2);
                        $mes = $this->getProximoMes(substr($dtBase, 3, 2));
                        $ano = substr($dtBase, 6, 4);
                        if(checkdate($mes, $dia, $ano)) {
                            $dtAux = substr($dtBase, 0, 3) . $this->getProximoMes(substr($dtBase, 3, 2)) . substr($dtBase, 5, 5);
                        }
                        else {
                            $dtAux = $this->getUltimoDiaMes($mes, $ano) .  substr($dtBase, 2, 8);
                        }
                    }
                }
                if($dtAux && $dtBase) {
                    $nDias = $this->getDiferenciaDias($dtBase, $dtAux);
                }
            }
        }
    }
    
    protected function trataTipoDtPgto($idtipodtpgto, &$nDias, &$dtBase, $dtEntrada, $dtEmissao) {
        if($idtipodtpgto == 'EMIN') {
            $nDias = 0;
            $dtBase = $dtEmissao;
        }
        else if($idtipodtpgto == 'EMEX') {
            $nDias = 1;
            $dtBase = $dtEmissao;
        }
        else if($idtipodtpgto == 'ENIN' || $idtipodtpgto == 'DSIN') {
            $nDias = 0;
            $dtBase = $dtEntrada;                    
        }
        else if($idtipodtpgto == 'ENEX' || $idtipodtpgto == 'DSEX') {
            $nDias = 1;
            $dtBase = $dtEntrada;
        }else{
            $nDias = 0;
            $dtBase = $dtEntrada;
        }
    }
    
    protected function getDataFeriado($ferifeli, &$dtParcela, $cdfilemit) {
        $nrDias = $ferifeli["NRDIAS"];
        $testaFeriado = true;
        while($testaFeriado) {
            $dtAnterior = $dtParcela;

            $dtParcela  = $this->adicionaDiasData($dtParcela, $nrDias);
            $ferifeli = $this->factory->getFeriFili($cdfilemit, $dtAnterior, $dtParcela);  

            if($ferifeli) {
                if($ferifeli["NRDIAS"] > 0) {
                    $nrDias = $ferifeli["NRDIAS"];
                    $testaFeriado = true;
                }
                else {
                    $testaFeriado = false;
                }
            }
            else {
                $testaFeriado = false;
            }
        }
    }
    
    protected function diaPgto($cdFormPgto, &$dtAux, $tipo) {
        $diaBase = substr($dtAux, 0, 2);

        if($tipo == 'S') {
            $diaBase = $this->getDiaSemana($dtAux);
        }
        else {
            $diaBase = substr($dtAux, 0, 2);
        }

        $diasPgto = $this->factory->getDiasPgto($cdFormPgto);
        
        foreach($diasPgto as $diaPgto) {
            $diaInicial = $diaPgto["NRDIAINIPGTO"];
            $diaFinal   = $diaPgto["NRDIAFINPGTO"];
            if($diaInicial <= $diaFinal) {
                if($diaBase >= $diaInicial && $diaBase <= $diaFinal) {
                    $this->trataDataPorTipo($diaPgto, $dtAux, $tipo);    
                    break;
                }
            }
            else {
                if($diaBase >= $diaInicial || $diaBase <= $diaFinal) {
                    $this->trataDataPorTipo($diaPgto, $dtAux, $tipo);  
                    break;
                } 
            }
        }
    }
    
    protected function trataDataPorTipo($diaPgto, &$dtAux, $tipo) {
        if($tipo == 'M') {
            $this->trataDiaMes($diaPgto, $dtAux);
        }
        else if($tipo == 'S') {
            $this->trataDiaSemana($diaPgto, $dtAux);
        }
        else {
            $this->trataDiaSemanaMes($diaPgto, $dtAux);
        } 
    }
    
    private function trataDiaMes($diaPgto, &$dtAux) {

        if($diaPgto["NRDIAPGTOVAR"]) {
            if($diaPgto["NRMESFORMPGT"] != 0) {
                $dtAux = $this->somaMesesData($dtAux, $diaPgto["NRNUMMESPGTO"]);
            } 
            
            $mes = substr($dtAux, 3, 2);
            $ano = substr($dtAux, 6, 4);
            
            if(checkdate($mes, $diaPgto["NRDIAPGTOVAR"], $ano)) {
                $dtAux = str_pad($diaPgto["NRDIAPGTOVAR"], 2, "0", STR_PAD_LEFT) .  substr($dtAux, 2, 8);
            }
            else {
                $dtAux = $this->getUltimoDiaMes($mes, $ano) .  substr($dtAux, 2, 8);
            }
            if($diaPgto["NRDIAPGTOSEMANA"] != '00' && ((int) $diaPgto["NRDIAPGTOSEMANA"]) != 0) {
                $dia = (int) $diaPgto["NRDIAPGTOSEMANA"];
                while($this->getDiaSemana($dtAux) != $dia) {
                    $dtAux = $this->adicionaDiasData($dtAux, 1);
                }
            }
        }
    }
    
    private function trataDiaSemana($diaPgto, &$dtAux) {
        if($diaPgto["NRMESFORMPGT"] != 0) $this->adicionaDiasData($dtAux, 7);
        
        $dia = (int) $diaPgto["NRDIAPGTOVAR"];
        
        while($this->getDiaSemana($dtAux) != $dia) {
            $dtAux = $this->adicionaDiasData($dtAux, 1);
        }
    }
    
    
    protected function trataDiaSemanaMes($diaPgto, &$dtAux) {
        if($diaPgto["NRDIAPGTOVAR"]) {
            $dtAux = "01" . substr($dtAux, 2, 8);
            if($diaPgto["NRMESFORMPGT"] != "0") {
                $dtAux = $this->somaMesesData($dtAux, $diaPgto["NRMESFORMPGT"]);
            }
            
            if($diaPgto["NRNUMMESPGTO"] == 2) {
                $this->adicionaDiasData($dtAux, 7);
            }
            else if($diaPgto["NRNUMMESPGTO"] == 3) {
                $this->adicionaDiasData($dtAux, 14);
            }
            else if($diaPgto["NRNUMMESPGTO"] == 4) {
                $this->adicionaDiasData($dtAux, 21);
            }
          
            $dia = (int) $diaPgto["NRDIAPGTOVAR"];

            while($this->getDiaSemana($dtAux) != $dia) {
                $dtAux = $this->adicionaDiasData($dtAux, 1);
            }
        }
    }
    /**
     * Funções auxiliares de data 
     */
    protected function somaMesesData($data, $nMes) {

        $mes = substr($data, 3, 2);
        for($i = 1; $i <= $nMes; $i++) {
            $mes = $this->getProximoMes($mes);
        }
        
        $data = substr($data, 0, 3) . $mes . substr($data, 5, 5);
        
        return $data;
    }
     
    protected function getProximoMes($mes) {

        if($mes != "12") {
            return str_pad(((int) $mes + 1), 2, "0", STR_PAD_LEFT);
        }
        return "01";
    }
    
    protected function getUltimoDiaMes($mes, $ano) {
        return date("t", mktime(0,0,0,$mes,'01',$ano));
    }
    
    protected function getDiaSemana($data) {
        $data = str_replace("/", "-", $data);
        return date('w', strtotime($data)) + 1;
    }
    
    protected function adicionaDiasData($data, $nDias, $sub = false) {
        if($nDias == 0) return $data;
        if($nDias <  0) $sub = true;
        
        if(gettype($data) == "string") {
            $date = $this->getDateFromString($data);
        }
        
        if($sub === false) {
            return $date->add(new \DateInterval('P'.$nDias.'D'))->format('d/m/Y');;
        }
        else {
            return $date->sub(new \DateInterval('P'.abs($nDias).'D'))->format('d/m/Y');;
        }

    }
    
    protected function getDiferenciaDias($data1, $data2) {
        
        $data1 = str_replace("/", "-", $data1);
        $data2 = str_replace("/", "-", $data2);
        
        $d1 = strtotime($data1); 
        $d2 = strtotime($data2);

        $dataFinal = ($d2 - $d1) / 86400;
        
        return $dataFinal;
    }
    
    /**
     * Funções para inserção do vencimento no banco
     */
    public function insereVencimentos($dataSourceName, $vencimentos) {
        if(!empty($vencimentos)) {
            $this->factory->deletaVencimentos($dataSourceName, $vencimentos[0]);
            $dataSource = $this->nameProvider->getDataSourceByName($dataSourceName);
            $primaryKeys = $dataSource->getPrimaryKeyColumns();
            
            foreach($vencimentos as &$vencimento) {
                $this->adicionaCamposAuditoria($vencimento);
                $vencimento = $this->getRowWithDefaultColumnsProperties($vencimento, $dataSource);
            }
            
            return $this->crudService->save($dataSourceName, $vencimentos);
        }

    }
    
    private function getRowWithDefaultColumnsProperties($row, $dataSource) {
        if (method_exists($dataSource, 'getColumnsProperties')) {
            $columnsProperties = $dataSource->getColumnsProperties();

            if (isset($columnsProperties)) {
                foreach ($columnsProperties as $columnProperty) {
                    if (!array_key_exists($columnProperty['COLUMN_NAME'], $row)) {
                        if ($columnProperty['NULLABLE'] == 'Y') {
                            $row[$columnProperty['COLUMN_NAME']] = $this->getDataDefaultValue($columnProperty);
                        } else if ($columnProperty['COLUMN_NAME'] == 'CDFILIAL') {
                            $row[$columnProperty['COLUMN_NAME']] = $this->environment->getCdFilial();
                        } else if ($columnProperty['NULLABLE'] == 'N') {
                            if (isset($columnProperty['DATA_DEFAULT'])){
                                $row[$columnProperty['COLUMN_NAME']] = $this->getDataDefaultValue($columnProperty);
                            } else {
                                throw new \Exception("Column ".$columnProperty['COLUMN_NAME']." cannot be null and does not have a default value! Set a value for this column.", 1);
                            }
                        }
                    }
                }
            }
        }

        return $row;
    }
    
    private function getDataDefaultValue($columnProperty) {
        if ($columnProperty['DATA_DEFAULT'] == null) {
            return null;
        } else {
            switch ($columnProperty['DATA_TYPE']) {
                case 'DATE':
                    return $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
                    break;
                case 'NUMBER':
                    return intval($columnProperty['DATA_DEFAULT']);
                    break;
                case 'FLOAT':
                    return floatval($columnProperty['DATA_DEFAULT']);
                    break;
                default:
                    return $columnProperty['DATA_DEFAULT'];
                    break;
            }
        }
    }
    
    protected function adicionaCamposAuditoria(&$row) {
        $currentTime = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
        $row['NRORG']       = (isset($row['NRORG']) && !empty($row['NRORG'])) ? $row['NRORG'] : $this->environment->getNrOrgTrab();
        $row['IDATIVO']     = isset($row['IDATIVO']) ? ($row['IDATIVO'] ?: 'S') : 'S';
        $row['DTULTATU']    = $currentTime;
        if($row["__is_new"] == true) {
            $row['DTINCLUSAO']      = $currentTime;
            $row['NRORGINCLUSAO']   = $this->environment->getNrOrgTrab();
            $row['CDOPERINCLUSAO']  = $this->environment->getCdOperador();
        }
        $row['NRORGULTATU']     = $this->environment->getNrOrgTrab();
        $row['CDOPERULTATU']    = $this->environment->getCdOperador();
        $row['DTULTATU']        = $currentTime;
    }
    
    protected function getDateFromString($data) {
        if (!$data) {
            throw new \Exception ('Data para cálculo de vencimento em branco.');
        }
        $date = \DateTime::createFromFormat('d/m/Y', $data);
        if($date == false){
            $date = \DateTime::createFromFormat('d/m/Y 00:00:00', $data);
        }
        if($date == false) {
            $d=strtotime($data);
            if ($d == false) {
                throw new \Exception ('Formato de Data para cálculo de vencimento inválido: ' . $data);
            }
            $date = \DateTime::createFromFormat('U', $d);
        }
        return $date;
    }
    
}